const path = require('path');
const path2 = require('path');
const fs = require('fs');
const ora = require('ora');
const co = require('co');
const prompt = require('co-prompt');
const chalk = require('chalk');
const templates = require("../templates");
const util = require('./util.js')
let chooseNum = null
let gitUrl = null
let list = null

const getList = async function () {
    return await templates.getList();
}
/**
 * 主逻辑代码
 */
const generator = function* () {
    // 模板名
    let tempName = null
    // 远程仓库地址
    let path = null;
    console.log('    可用模板列表:')
    list = yield getList()
    for (let key in list) {
        let tempName = list[key].name
        console.log(
            '    ' + chalk.green(key) +
            ' : ' + chalk.green(tempName))
    }
    // 询问选择第几项
    chooseNum = yield prompt("    请选择模板类型(0): ")
    if (!chooseNum) {
        chooseNum = 0
    }
    // 选择列表第几项后得到git仓库的地址
    path = list[chooseNum].path
    // 如果地址错误就异常并且退出
    if (path) {
        console.log('    ----------------------------------------')
        let projectName = yield prompt("    请输入项目名称(demo):")
        if (!projectName) {
            projectName = "demo"
        }
        console.log('    ----------------------------------------')

        gitUrl = yield prompt("    请输入项目git地址:")
        console.log('    ----------------------------------------')
        co(downloadTemplates(path, projectName));
    } else {
        console.log(chalk.red(`   模版[${tempName}]不存在或地址错误`))
        process.exit(0);
    }
}

/**
 * 下载仓库
 * @param {*} path 
 * @param {*} projectName 
 */
function* downloadTemplates(path, projectName) {
    let spanner = ora("  正在下载项目，请稍等......");
    spanner.start();
    const sourcePath = path2.resolve(process.cwd(), projectName)
    if (fs.existsSync(sourcePath)) {
        //刪除原文件
        util.rmdirSync(sourcePath);
    }
    require("child_process").exec(`git clone ${path} ${sourcePath}`, function (err, stdout, stderr) {
        if (err) {
            spanner.stop();
            console.log('   ', chalk.red('项目构建失败'), err);
            process.exit(0);
        }
        co(startBuildProject(spanner, projectName))
    })
}

/**
 * 从download目录复制到最终的目录，执行初始化脚本
 * @param {*} spanner 等待工具类
 * @param {*} projectName 项目名
 */
function* startBuildProject(spanner, projectName) {
    console.log('                                               ')
    console.log('   ', chalk.green('*'), chalk.green('下载项目成功'));
    spanner.stop();
    let spanner1 = ora("  正在初始化项目，请稍等......");
    spanner1.start();
    require("child_process").exec(`cd ${projectName} && ${list[chooseNum].initScripts}`, function (err, stdout, stderr) {
        if (err) {
            spanner1.stop();
            console.log('                                               ')
            console.log('   ', chalk.red('初始化失败'), err);
            process.exit(0);
        }
        console.log('                                               ')
        //init项目
        co(rebuildGitOrigin(spanner1, projectName))
    })
}

/**
 * 新项目，不需要历史提交。框架维护对于开发者应该是个黑盒
 */
 function* rebuildGitOrigin(spanner, projectName) {
    require("child_process").exec(`cd ${projectName} && rm -rf ./.git && git init && git add . && git commit -m "init;"`, function (err, stdout, stderr) {
        if (err) {
            spanner.stop();
            console.log('                                               ')
            console.log('   ', chalk.red('git init失败'), err);
            process.exit(0);
        }
        console.log('                                               ')
        console.log('   ', chalk.green('*'), chalk.green('git init成功'));
        spanner.stop();
        if (gitUrl) {
            let spanner2 = ora("  正在修改项目远程仓库，请稍等......");
            spanner2.start();
            co(changeGitOrigin(spanner2, projectName))
        } else {
            let spanner2 = ora("  正在安装项目，请稍等......");
            spanner2.start();
            co(installyProject(spanner2, projectName))
        }
    })
}
/**
 * 修改git远程仓库地址origin
 */
function* changeGitOrigin(spanner, projectName) {
    require("child_process").exec(`cd ${projectName} && git remote add origin ${gitUrl};`, function (err, stdout, stderr) {
        if (err) {
            spanner.stop();
            console.log('                                               ')
            console.log('   ', chalk.red('git远程仓库地址修改失败'), err);
            process.exit(0);
        }
        console.log('                                               ')
        console.log('   ', chalk.green('*'), chalk.green('git远程仓库地址修改成功'));
        spanner.stop();
        let spanner2 = ora("  正在推送项目远程仓库，请稍等......");
        spanner2.start();
        co(pushGitOrigin(spanner2, projectName))
    })
}

/**
 * 推送origin
 */
 function* pushGitOrigin(spanner, projectName) {
    require("child_process").exec(`cd ${projectName} && git push -u origin master;`, function (err, stdout, stderr) {
        if (err) {
            spanner.stop();
            console.log('                                               ')
            console.log('   ', chalk.red('推送git远程仓库失败，需要手动拉取合并推送'), err);
            process.exit(0);
        }
        console.log('                                               ')
        console.log('   ', chalk.green('*'), chalk.green('推送git远程仓库地址'));
        spanner.stop();
        let spanner2 = ora("  正在安装项目，请稍等......");
        spanner2.start();
        co(installyProject(spanner2, projectName))
    })
}

/**
 * 安装依赖
 * @param {*} spanner 等待工具类
 * @param {*} projectName 项目名
 */
function* installyProject(spanner, projectName) {
    require("child_process").exec(`cd ${projectName} && ${list[chooseNum].packageScripts}`, function (err, stdout, stderr) {
        if (err) {
            spanner.stop();
            console.log('                                               ')
            console.log('   ', chalk.red('安装依赖失败'), err);
            process.exit(0);
        }
        console.log('                                               ')
        console.log('   ', chalk.green('*'), chalk.green('安装依赖成功'));
        spanner.stop();
        process.exit(0);
    })
}

// bin的init方法,先执行获取版本信息，再通过co、yield执行具体逻辑
module.exports = function () {
    co(generator());
}